#include "bon_time.h"
#include <stdio.h>
#include <sys/time.h>
#ifdef SysV
#include <limits.h>
#include <sys/times.h>
#else
#include <sys/resource.h>
#endif
#include <unistd.h>
#include <time.h>


void BonTimer::timestamp()
{
  m_last_timestamp = time_so_far();
  m_last_cpustamp = cpu_so_far();
}

void BonTimer::get_delta_t(tests_t test)
{
  m_delta[test].Elapsed = time_so_far() - m_last_timestamp;
  m_delta[test].CPU = cpu_so_far() - m_last_cpustamp;
  timestamp();
}

void BonTimer::get_delta_report(report_s &rep)
{
  rep.EndTime = time_so_far();
  rep.CPU = cpu_so_far() - m_last_cpustamp;
  timestamp();
}

void BonTimer::add_delta_report(report_s &rep, tests_t test)
{
  if(m_delta[test].CPU == 0.0)
  {
    m_delta[test].FirstStart = rep.StartTime;
    m_delta[test].LastStop = rep.EndTime;
  }
  else
  {
    if(rep.StartTime < m_delta[test].FirstStart)
      m_delta[test].FirstStart = rep.StartTime;
    if(rep.EndTime > m_delta[test].LastStop)
      m_delta[test].LastStop = rep.EndTime;
  }
  m_delta[test].CPU += rep.CPU;
  m_delta[test].Elapsed = m_delta[test].LastStop - m_delta[test].FirstStart;
}

double
BonTimer::cpu_so_far()
{
#ifdef SysV
  struct tms tms;
 
  if (times(&tms) == -1)
    io_error("times", true);
  return ((double) tms.tms_utime) / ((double) CLK_TCK) +
    ((double) tms.tms_stime) / ((double) CLK_TCK);
 
#else
  struct rusage rusage;
 
  getrusage(RUSAGE_SELF, &rusage);
  return
    ((double) rusage.ru_utime.tv_sec) +
      (((double) rusage.ru_utime.tv_usec) / 1000000.0) +
        ((double) rusage.ru_stime.tv_sec) +
          (((double) rusage.ru_stime.tv_usec) / 1000000.0);
#endif
}

double
BonTimer::time_so_far()
{
#ifdef SysV
  int        val;
  struct tms tms;
 
  if ((val = times(&tms)) == -1)
    io_error("times", true);
 
  return ((double) val) / ((double) CLK_TCK);
 
#else
  struct timeval tp;
 
  if (gettimeofday(&tp, (struct timezone *) NULL) == -1)
    io_error("gettimeofday", true);
  return ((double) (tp.tv_sec - m_basetime)) +
    (((double) tp.tv_usec) / 1000000.0);
#endif
}

BonTimer::BonTimer()
{
  for(int i = 0; i < TestCount; i++)
  {
    m_delta[i].CPU = 0.0;
    m_delta[i].Elapsed = 0.0;
  }
  m_basetime = (int) time((time_t *) NULL);
  timestamp();
  m_type = txt;
  m_first_time = true;
}

int BonTimer::print_cpu_stat(tests_t test)
{
  if(m_delta[test].Elapsed == 0.0)
  {
    if(m_type == txt)
      printf("     ");
    else
      printf(",");
    return 0;
  }
  double cpu = m_delta[test].CPU / m_delta[test].Elapsed * 100.0;
  if(m_type == txt)
  {
    if(cpu > 99.9)
      printf(" 100.");
    else
      printf(" %4.1f", cpu);
  }
  else
    printf(",%.1f", cpu);
  return 0;
}

int BonTimer::print_io_stat(tests_t test)
{
  if(m_delta[test].Elapsed == 0.0)
  {
    if(m_type == txt)
      printf("      ");
    else
      printf(",");
  }
  else
  {
    int stat = (int) (((double) m_file_size) / (m_delta[test].Elapsed / 1024.0));
    if(m_type == txt)
    {
      if(m_delta[test].Elapsed < 1.0)
        printf(" +++++");
      else
        printf(" %5d", stat);
    }
    else
      printf(",%d", stat);
  }
  print_cpu_stat(test);
  return 0;
}

int BonTimer::print_seek_stat(tests_t test)
{
  double seek_stat = ((double) Seeks) / m_delta[test].Elapsed;
  if(m_type == txt)
  {
    if(m_delta[test].Elapsed == 0.0)
      printf("      ");
    else if(m_delta[test].Elapsed < 1.0)
      printf(" +++++");
    else
      printf(" %5.1f", seek_stat);
  }
  else
  {
    if(m_delta[test].Elapsed == 0.0)
      printf(",");
    else if(m_delta[test].Elapsed < 1.0)
      printf(",+++++");
    printf(",%5.1f", seek_stat);
  }
  print_cpu_stat(test);
  return 0;
}
 
int BonTimer::print_file_stat(tests_t test)
{
  int stat = (int) (((double) m_directory_size) / m_delta[test].Elapsed);
  if(m_type == txt)
  {
    if(m_delta[test].Elapsed == 0.0)
      printf("      ");
    else if(m_delta[test].Elapsed < 1.0)
      printf(" +++++");
    else
      printf(" %5d", stat);
  }
  else
  {
    if(m_delta[test].Elapsed == 0.0)
      printf(",");
    else if(m_delta[test].Elapsed < 1.0)
      printf(",+++++");
    else
      printf(",%d", stat);
  }
  print_cpu_stat(test);
  return 0;
}

void
BonTimer::DoReport(CPCCHAR machine, int file_size, int directory_size)
{
  m_file_size = file_size;
  m_directory_size = directory_size;
  if(m_type == csv)
  {
    if(m_first_time)
    {
      m_first_time = false;
      printf("name");
      printf(",file_size,putc,putc_cpu,put_block,put_block_cpu,rewrite,rewrite_cpu,getc,getc_cpu,get_block,get_block_cpu,seeks,seeks_cpu");
      printf(",num_files,seq_create,seq_create_cpu,seq_stat,seq_stat_cpu,seq_del,seq_del_cpu,ran_create,ran_create_cpu,ran_stat,ran_stat_cpu,ran_del,ran_del_cpu");
      printf("\n");
    }
  }
  if(m_file_size)
  {
    if(m_type == txt)
    {
      printf("Version %5s ", BON_VERSION);
      printf(
        "-------Sequential Output-------- ---Sequential Input-- --Random--\n");
      printf("              ");
      printf(
        "-Per Char- --Block--- -Rewrite-- -Per Char- --Block--- --Seeks---\n");
      printf("Machine    MB ");
      printf("K/sec %%CPU K/sec %%CPU K/sec %%CPU K/sec %%CPU K/sec ");
      printf("%%CPU  /sec %%CPU\n");
    }
    if(m_type == txt)
      printf("%-8.8s %4d", machine, m_file_size);
    else
      printf("%s,%d", machine, m_file_size);
    print_io_stat(Putc);
    print_io_stat(FastWrite);
    print_io_stat(ReWrite);
    print_io_stat(Getc);
    print_io_stat(FastRead);
    print_seek_stat(Lseek);
    if(m_type == txt)
      printf("\n");
  }
  else if(m_type == csv)
  {
    printf("%s,,,,,,,,,,,,,", machine);
  }

  if(m_directory_size)
  {
    if(m_type == txt)
    {
      if(m_file_size)
        printf("              ");
      else
        printf("Version %5s ", BON_VERSION);
      printf(
        "-------Sequential Create-------- ----------Random Create---------\n");
      printf("        files ");
      printf(
           "--Create-- ---Stat--- --Delete-- --Create-- ---Stat--- --Delete--\n");
      printf("       %6d ", m_directory_size);
      printf(" /sec %%CPU  /sec %%CPU  /sec %%CPU  /sec %%CPU  /sec ");
      printf("%%CPU  /sec %%CPU\n");
      printf("             ");
    }
    else
    {
      printf(",%d", m_directory_size);
    }
    print_file_stat(CreateSeq);
    print_file_stat(StatSeq);
    print_file_stat(DelSeq);
    print_file_stat(CreateRand);
    print_file_stat(StatRand);
    print_file_stat(DelRand);
    printf("\n");
  }
  else if(m_type == csv)
  {
    printf(",,,,,,,,,,,,,\n");
  }
}

